package thesis.algorithms.evolutionary.bivariate;

import java.util.Arrays;

import thesis.algorithms.evolutionary.bivariate.dependecy_graphs.IDependencyGraph;
import thesis.algorithms.evolutionary.bivariate.probability_matrix.IProbabilityMatrix;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.Chromosome;

public class BDMA implements IAlgorithm {

	private Chromosome[] population;
	private IFunction function;
	private IDecoder decoder;
	private int promisingSolutionsNumber;
	private IProbabilityMatrix probabilityMatrix;
	private IDependencyGraph dependencyGraph;
	private double[][] chiSquare;

	public BDMA(IDecoder decoder, IFunction function,
			int promisingSolutionNumber, int populationSize,
			IDependencyGraph dependencyGraph,
			IProbabilityMatrix probabilityMatrix) {

		this.decoder = decoder;
		this.function = function;
		this.promisingSolutionsNumber = promisingSolutionNumber;
		this.dependencyGraph = dependencyGraph;
		this.probabilityMatrix = probabilityMatrix;

		population = new Chromosome[populationSize];
		chiSquare = new double[decoder.getBitsNumber()][decoder.getBitsNumber()];
	}

	@Override
	public void initializePopulation() {

		for (int i = 0; i < population.length; ++i) {
			if (population[i] == null) {
				population[i] = new Chromosome();
			}
		}

		for (int i = 0; i < population.length; ++i) {
			population[i].sample(decoder.getBitsNumber(),
					probabilityMatrix.getProbabilityVector());
		}
	}

	@Override
	public void iterate() {

		for (int i = 0; i < population.length; ++i) {
			population[i].evaluate(decoder, function);
		}
		
		Arrays.sort(population);
		System.out.println(population[0].getFitnessValue());
	
		probabilityMatrix.updateProbabilityMatrix(population,
				promisingSolutionsNumber);
		fillChiSquare();
		
		dependencyGraph.constructDependencyGraph(chiSquare);
		
		samplePopulation();
	}

	private void fillChiSquare() {

		for (int row = 0; row < decoder.getBitsNumber(); ++row) {
			for (int column = row + 1; column < decoder.getBitsNumber(); ++column) {

				double denomerator = probabilityMatrix.getProbability(row)
						* probabilityMatrix.getProbability(column);
				double numerator = probabilityMatrix.getJointProbability(row,
						column) - denomerator;
				numerator = numerator * numerator;

				chiSquare[row][column] = (numerator / denomerator)
						* population.length;
				chiSquare[column][row] = chiSquare[row][column];
			}
		}

	}

	private void samplePopulation() {

		for (int i = 1; i < population.length; ++i) {

			population[i] = dependencyGraph
					.generateNewIndividual(probabilityMatrix);
		}
	}
}
