package thesis.algorithms.evolutionary.bivariate.probability_matrix;

import java.util.Arrays;

import thesis.algorithms.evolutionary.univariate.probability_vector.IProbabilityVector;
import thesis.algorithms.evolutionary.univariate.probability_vector.UMDAProbabilityVector;
import thesis.util.Chromosome;

public class BMDAProbabilityMatrix implements IProbabilityMatrix {

	private IProbabilityVector vector;
	private double[][] probabilityMatrix;
	private int[][] totalBitsNumber;
	private int[][] totalOnesNumber;

	private int bitsNo;

	public BMDAProbabilityMatrix(int bitsNo) {

		vector = new UMDAProbabilityVector(bitsNo);
		probabilityMatrix = new double[bitsNo][bitsNo];
		totalBitsNumber = new int[bitsNo][bitsNo];
		totalOnesNumber = new int[bitsNo][bitsNo];

		this.bitsNo = bitsNo;
	}

	@Override
	public void updateProbabilityMatrix(Chromosome[] individuals,
			int promisingSolutionsNumber) {

		vector.updateProbabilityVector(Arrays.copyOf(individuals,
				promisingSolutionsNumber));

		cleanBitsCount();

		for (int i = 0; i < promisingSolutionsNumber; ++i) {
			updateProbabilityMatrix(individuals[i]);
		}

	}

	private void cleanBitsCount() {
		for (int i = 0; i < bitsNo; ++i) {
			for (int j = 0; j < bitsNo; ++j) {
				totalBitsNumber[i][j]=0;
				totalOnesNumber[i][j]=0;
			}
		}

	}

	private void updateProbabilityMatrix(Chromosome chromosome) {

		for (int i = 0; i < chromosome.getLength(); ++i) {
			for (int j = i + 1; j < chromosome.getLength(); ++j) {

				++totalBitsNumber[i][j];
				if (chromosome.getBit(i) && chromosome.getBit(j)) {
					++totalOnesNumber[i][j];
				}
			}
		}

		for (int i = 0; i < chromosome.getLength(); ++i) {
			for (int j = i + 1; j < chromosome.getLength(); ++j) {
				probabilityMatrix[i][j] = (double) totalOnesNumber[i][j]
						/ (double) totalBitsNumber[i][j];
				probabilityMatrix[j][i] = probabilityMatrix[i][j];
			}
		}
	}

	@Override
	public double getJointProbability(int i, int j) {

		return probabilityMatrix[i][j];
	}

	@Override
	public double[][] getProbabilities() {

		return probabilityMatrix;
	}

	@Override
	public double[] getProbabilityVector() {

		return vector.getProbabilityVector();
	}

	@Override
	public double getProbability(int i) {
		return vector.getProbabilityVector()[i];
	}

}
