package thesis.algorithms.evolutionary.genetic;

import java.util.Arrays;

import thesis.algorithms.evolutionary.genetic.operators.ICrossing;
import thesis.algorithms.evolutionary.genetic.operators.IMutation;
import thesis.algorithms.evolutionary.genetic.operators.ISelection;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.Chromosome;

public class GeneticAlgorithm implements IAlgorithm {
	private Chromosome[] population;
	
	private ISelection selection;
	private ICrossing crossing;
	private IMutation mutation;
	private IFunction function;
	private IDecoder decoder;

	private double currentMinimum = Double.MAX_VALUE;

	public GeneticAlgorithm(IMutation mutation, ICrossing crossing,
			ISelection selection, IDecoder decoder, IFunction function,
			int populationSize) {

		this.function = function;
		this.decoder = decoder;
		this.mutation = mutation;
		this.crossing = crossing;
		this.selection = selection;

		population = new Chromosome[populationSize];

	}

	@Override
	public void initializePopulation() {
		int totalBits = decoder.getBitsNumber();

		for (int i = 0; i < population.length; ++i) {
			population[i] = new Chromosome(totalBits);
			population[i].evaluate(decoder, function);
		}
	}

	@Override
	public void iterate() {

		Chromosome[] parents = selection.selection(population);

		crossing.crossing(parents);
		mutation.mutation(parents[2]);

		for (int i = 0; i < parents.length; ++i) {
			parents[i].evaluate(decoder, function);
		}
		Arrays.sort(population);
		currentMinimum = population[0].getFitnessValue();

		System.out.println(" Current minimum is: " + currentMinimum);
	}
}
