package thesis.algorithms.evolutionary.univariate;

import java.util.Arrays;

import thesis.algorithms.evolutionary.univariate.probability_vector.CGAProbabilityVector;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.Chromosome;

public class CompactGA implements IAlgorithm {

	private Chromosome[] population;
	private IDecoder decoder;
	private IFunction function;
	private CGAProbabilityVector probabilityVector;

	public CompactGA(int populationSize, double learningRate, IDecoder decoder,
			IFunction function) {

		this.function = function;
		this.decoder = decoder;

		this.probabilityVector = new CGAProbabilityVector(
				decoder.getBitsNumber(), 1 / populationSize);
		population = new Chromosome[populationSize];

	}

	@Override
	public void initializePopulation() {

		for (int i = 0; i < population.length; i++) {
			population[i] = new Chromosome(decoder.getBitsNumber());
			population[i].sample(decoder.getBitsNumber(),
					probabilityVector.getProbabilityVector());
			population[i].evaluate(decoder, function);
		}

	}

	@Override
	public void iterate() {

		Arrays.sort(population);
		System.out.println(population[0].getFitnessValue());
		
		population[population.length - 2].sample(decoder.getBitsNumber(),
				probabilityVector.getProbabilityVector());

		population[population.length - 1].sample(decoder.getBitsNumber(),
				probabilityVector.getProbabilityVector());

		Chromosome[] sampled = new Chromosome[2];
		sampled[0] = population[population.length - 2];
		sampled[1] = population[population.length - 1];

		sampled[0].evaluate(decoder, function);
		sampled[1].evaluate(decoder, function);

		Arrays.sort(sampled);

		probabilityVector.updateProbabilityVector(sampled);
	}

}
