package thesis.algorithms.evolutionary.univariate;

import java.util.Arrays;

import thesis.algorithms.evolutionary.univariate.probability_vector.PBILProbabilityVector;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.Chromosome;

public class PBIL implements IAlgorithm {
	private Chromosome[] population;
	private IDecoder decoder;
	private IFunction function;
	private PBILProbabilityVector probabilityVector;

	public PBIL(int populationSize, double mutationStep, double learningRate,
			IDecoder decoder, IFunction function) {

		this.function = function;
		this.probabilityVector = new PBILProbabilityVector(decoder.getBitsNumber(),
				learningRate, mutationStep);
		population = new Chromosome[populationSize];
		this.decoder = decoder;
	}

	private void samplePopulation() {
		
		for (int i = 0; i < population.length; i++) {
			population[i].sample(decoder.getBitsNumber(),
					probabilityVector.getProbabilityVector());
		}
	}

	@Override
	public void initializePopulation() {
		
		for (int i = 0; i < population.length; i++) {
			if (population[i] == null) {
				population[i] = new Chromosome();
			}
		}
	}

	@Override
	public void iterate() {
		
		samplePopulation();
		for (int i = 0; i < population.length; i++) {
			population[i].evaluate(decoder, function);
		}
		
		Arrays.sort(population);
		System.out.println(population[0].getFitnessValue());
		probabilityVector.updateProbabilityVector(Arrays.copyOf(population, 1));
	}

}
