package thesis.algorithms.evolutionary.univariate;

import java.util.Arrays;

import thesis.algorithms.evolutionary.univariate.probability_vector.UMDAProbabilityVector;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.Chromosome;

public class UMDA implements IAlgorithm {

	private Chromosome[] population;
	private UMDAProbabilityVector probabilityVector;
	
	private IFunction function;
	private IDecoder decoder;
	private int promisingSolutionsNumber;
	
	
	public UMDA(int populationSize, int promisingSolutions, IFunction function,
			IDecoder decoder) {
		this.promisingSolutionsNumber = promisingSolutions;
		this.function = function;
		this.decoder = decoder;

		population = new Chromosome[populationSize];
		probabilityVector = new UMDAProbabilityVector(decoder.getBitsNumber());
	}

	private void samplePopulation() {

		for (int i = 0; i < population.length; i++) {
			population[i].sample(decoder.getBitsNumber(),
					probabilityVector.getProbabilityVector());
		}
	}

	@Override
	public void initializePopulation() {

		for (int i = 0; i < population.length; i++) {
			if (population[i] == null) {
				population[i] = new Chromosome();
			}
		}
	}

	@Override
	public void iterate() {
		samplePopulation();
		for (int i = 0; i < population.length; i++) {
			population[i].evaluate(decoder, function);
		}
		
		Arrays.sort(population);
		System.out.println(population[0].getFitnessValue());
		probabilityVector.updateProbabilityVector(Arrays.copyOfRange(population, 0, promisingSolutionsNumber));

	}

}
