package thesis.algorithms.evolutionary.univariate.probability_vector;

import java.util.Arrays;

import thesis.util.Chromosome;

public class CGAProbabilityVector implements IProbabilityVector {
	private double[] vector;
	private double learningRate;

	public CGAProbabilityVector(int vectorSize, double learningRate) {
		vector = new double[vectorSize];
		Arrays.fill(vector, 0.5);

		this.learningRate = learningRate;
	}

	@Override
	public void updateProbabilityVector(Chromosome[] individuals) {

		boolean[] winner = individuals[0].getBinaryRepresentation();
		boolean[] looser = individuals[1].getBinaryRepresentation();

		for (int i = 0; i < vector.length; ++i) {

			// if bits are the same, do nothing
			if (winner[i] == looser[i]) {
				continue;
			}

			// else update values
			// if winners bit is true, increase probability
			if (winner[i]) {
				vector[i] = vector[i] + learningRate;
				if (vector[i] > 1) {
					vector[i] = 1;
				}
			}

			// if winners bit is false, decrease probability
			if (!winner[i]) {
				vector[i] = vector[i] - learningRate;
				if (vector[i] < 0) {
					vector[i] = 0;
				}
			}
		}

	}

	@Override
	public double[] getProbabilityVector() {
		return vector;
	}

}
