package thesis.algorithms.evolutionary.univariate.probability_vector;

import java.util.Arrays;
import java.util.Random;

import thesis.util.Chromosome;

public class PBILProbabilityVector implements IProbabilityVector {
	private double[] vector;
	private double learningRate;
	private double mutationStep;

	public PBILProbabilityVector(int vectorSize, double learningRate,
			double mutationStep) {

		vector = new double[vectorSize];
		Arrays.fill(vector, 0.5);

		this.learningRate = learningRate;
		this.mutationStep = mutationStep;
	}
	
	private void updateProbabilityVector(Chromosome bestIndividual) {
		
		for (int i = 0; i < vector.length; ++i) {
			int bit = 0;
			if (bestIndividual.getBit(i))
				bit = 1;
			vector[i] = vector[i] * (1 - learningRate) + bit * learningRate;
		}
	}
	
	private void mutateProbabilityVector() {
		
		Random random = new Random();

		for (int i = 0; i < vector.length; ++i) {
			int bit = random.nextInt(2);
			vector[i] = vector[i] * (1 - mutationStep) + bit * mutationStep;
		}
	}
	
	@Override
	public double[] getProbabilityVector() {
		
		return vector;
	}

	@Override
	public void updateProbabilityVector(Chromosome[] bestIndividuals) {

		for (int i = 0; i < bestIndividuals.length; ++i) {
			updateProbabilityVector(bestIndividuals[i]);
		}
		
		mutateProbabilityVector();
	}
}
