package thesis.executors;

import thesis.algorithms.evolutionary.bivariate.BDMA;
import thesis.algorithms.evolutionary.bivariate.dependecy_graphs.BMDADependencyGraph;
import thesis.algorithms.evolutionary.bivariate.probability_matrix.BMDAProbabilityMatrix;
import thesis.functions.RastriginFunction;
import thesis.interfaces.IAlgorithm;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.ChromosomeDecoder;

public class BDMAExecutor {

	private static final int xMin = -5;
	private static final int xMax = 5;
	private static final int dimensionNumber = 10;
	private static final double precision = 0.01;
	private static final int populationSize = 50;
	private static final int iterations = 1000;

	public static void main(String[] args) {
		
		IDecoder decoder = new ChromosomeDecoder(xMin, xMax, dimensionNumber,
				precision);
		IFunction function = new RastriginFunction();

		IAlgorithm algorithm = new BDMA(decoder, function, 10, populationSize,
				new BMDADependencyGraph(decoder.getBitsNumber()),
				new BMDAProbabilityMatrix(decoder.getBitsNumber()));

		algorithm.initializePopulation();
		for (int i = 0; i < iterations; i++) {
			algorithm.iterate();
		}

	}
}
