package thesis.executors;

import thesis.algorithms.evolutionary.genetic.GeneticAlgorithm;
import thesis.algorithms.evolutionary.genetic.operators.ICrossing;
import thesis.algorithms.evolutionary.genetic.operators.IMutation;
import thesis.algorithms.evolutionary.genetic.operators.ISelection;
import thesis.algorithms.evolutionary.genetic.operators.NormalMutation;
import thesis.algorithms.evolutionary.genetic.operators.ThreeTournamentSelection;
import thesis.algorithms.evolutionary.genetic.operators.UniformCrossover;
import thesis.functions.RastriginFunction;
import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;
import thesis.util.ChromosomeDecoder;

public class GAExecutor {

	private final static int iterationNumber = 5000;
	private final static int populationSize = 50;
	private final static double mutationProbability = 0.05;
	private final static double crossingProbability = 0.7;

	private final static double xMin = -10;
	private final static double xMax = 10;
	private final static double precision = 0.01;
	private final static int dimensionNumber = 10;

	public static void main(String[] args) {
		IFunction function = new RastriginFunction();
		IDecoder decoder = new ChromosomeDecoder(xMin, xMax, dimensionNumber,
				precision);
		ISelection selection = new ThreeTournamentSelection();
		IMutation mutation = new NormalMutation(mutationProbability);
		ICrossing crossing = new UniformCrossover(crossingProbability);

		GeneticAlgorithm ga = new GeneticAlgorithm(mutation, crossing,
				selection, decoder, function, populationSize);

		ga.initializePopulation();

		for (int i = 0; i < iterationNumber; i++) {
			ga.iterate();
		}
	}
}
