package thesis.util;

import java.util.Arrays;
import java.util.Random;

import thesis.interfaces.IDecoder;
import thesis.interfaces.IFunction;

public class Chromosome implements Comparable<Chromosome> {
	private double fitnessValue = Double.MAX_VALUE;
	private boolean[] binaryRepresentation = null;;

	public Chromosome() {

		super();
	}

	public Chromosome(int totalBits) {

		setSize(totalBits);
	}
	
	@Override
	public int compareTo(Chromosome o) {
		if (this.fitnessValue - o.fitnessValue < 0)
			return -1;
		if (this.fitnessValue - o.fitnessValue > 0)
			return 1;
		return 0;
	}

	public void evaluate(IDecoder decoder, IFunction function) {

		double[] values = decoder.decodeChromosome(this);
		fitnessValue = function.calculate(values);
	}

	public boolean[] getBinaryRepresentation() {
		return binaryRepresentation;
	}

	public int getLength() {

		return binaryRepresentation.length;
	}

	public boolean getBit(int index) {
		return binaryRepresentation[index];
	}

	public void setBit(int index, boolean value) {
		binaryRepresentation[index] = value;
	}

	public double getFitnessValue() {
		return fitnessValue;
	}

	public void setSize(int totalBits) {

		binaryRepresentation = new boolean[totalBits];
		Arrays.fill(binaryRepresentation, false);
	}

	public void sample(int totalBits, double[] probability) {

		binaryRepresentation = new boolean[totalBits];
		Random rand = new Random();

		for (int i = 0; i < totalBits; i++) {
			if (rand.nextDouble() < probability[i]) {
				binaryRepresentation[i] = true;
			} else {
				binaryRepresentation[i] = false;
			}
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null)
			return false;
		if (!(obj instanceof Chromosome))
			return false;
		Chromosome comparing = (Chromosome) obj;

		if (this.getLength() != comparing.getLength())
			return false;

		for (int i = 0; i < this.getLength(); i++) {
			if (binaryRepresentation[i] != comparing.getBit(i))
				return false;
		}

		return true;
	}
}
