package thesis.util;

import thesis.interfaces.IDecoder;

public class ChromosomeDecoder implements IDecoder {

	private int dimensionNumber;
	private double minValue;
	private double maxValue;
	private int bitsNumber;

	public ChromosomeDecoder(double xMin, double xMax, int dimensionNumber,
			double precision) {

		this.minValue = xMin;
		this.maxValue = xMax;
		this.dimensionNumber = dimensionNumber;

		bitsNumber = (int) Math.ceil(Math.log((xMax - xMin) / precision)/Math.log(2));
		bitsNumber *= dimensionNumber;
	}

	@Override
	public int getDimensionNumber() {
		return dimensionNumber;
	}

	@Override
	public double[] decodeChromosome(Chromosome chromosome) {
		int bitIndex = 0;
		int bits = bitsNumber / dimensionNumber;
		int biggestNum = (int) Math.pow(2, bits) - 1;

		boolean[] binCode = new boolean[chromosome.getLength()];
		boolean[] grayCode = chromosome.getBinaryRepresentation();
		for (int i = grayCode.length - 2; i >= 0; i--) {
			binCode[i] = binCode[i + 1];
			if (grayCode[i] == true && binCode[i] == false
					|| grayCode[i] == false && binCode[i] == true)
				binCode[i] = true;
			else
				binCode[i] = false;
		}

		double[] values = new double[dimensionNumber];
		for (int i = 0; i < dimensionNumber; i++) {
			double var = 0;
			int firstBit = bitIndex;
			int lastBit = firstBit + bits - 1;
			bitIndex += bits;
			for (int j = firstBit; j <= lastBit; j++) {
				var *= 2;
				if (binCode[j])
					var++;
			}

			values[i] = (var / biggestNum) * (maxValue - minValue) + minValue;
			
		}
		
		return values;
	}

	@Override
	public int getBitsNumber() {
		return bitsNumber;
	}

}
